@@ -1,29 +1,16 @@
-
 /*
- *  This file is part of the KDE Help Center
- *
- *  Copyright (c) 2002 Frerich Raabe <raabe@kde.org>
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
- */
+    SPDX-FileCopyrightText: 2002 Frerich Raabe <raabe@kde.org>
+
+    SPDX-License-Identifier: GPL-2.0-or-later
+*/
 
 #include "application.h"
 #include "mainwindow.h"
 
+#include <kwindowsystem_version.h>
 #include <KAboutData>
 #include <KDBusService>
+#include <KWindowSystem>
 #include <KLocalizedString>
 #include <KUrlAuthorized>
 
@@ -58,32 +45,69 @@ QCommandLineParser *Application::cmdParser()
   return &mCmdParser;
 }
 
-void Application::activate(const QStringList& args, const QString &workingDirectory)
+void Application::activateForStartLikeCall()
 {
-  mCmdParser.parse( args );
-
-  QStringList urls = mCmdParser.positionalArguments();
-
-  if( !mMainWindow )
-  {
-    if (qApp->isSessionRestored()) {
-        // The kRestoreMainWindows call will do the rest.
-        return;
-    }
-    mMainWindow = new MainWindow;
-  }
-
-  QUrl url;
-  if ( !urls.isEmpty() ) {
-    url = QUrl::fromUserInput( urls.at( 0 ), workingDirectory );
-  }
-
-  mMainWindow->openUrl( url );
-
   mMainWindow->show();
+#if KWINDOWSYSTEM_VERSION >= QT_VERSION_CHECK(5, 91, 0)
+  KWindowSystem::updateStartupId(mMainWindow->windowHandle());
+#else
+  if (KWindowSystem::isPlatformWayland()) {
+    const QString token = qEnvironmentVariable("XDG_ACTIVATION_TOKEN");
+    if (!token.isEmpty()) {
+      KWindowSystem::setCurrentXdgActivationToken(token);
+      // no need to unset XDG_ACTIVATION_TOKEN, because
+      // KDBusService cares for that and we only run in its signals handlers here
+    }
+  }
+#endif
+
+  mMainWindow->raise();
+  KWindowSystem::activateWindow(mMainWindow->winId());
 }
 
-extern "C" int Q_DECL_EXPORT kdemain( int argc, char **argv )
+void Application::newInstance()
+{
+  mMainWindow = new MainWindow;
+
+  mMainWindow->show();
+
+  const QStringList urls = mCmdParser.positionalArguments();
+  if ( !urls.isEmpty() ) {
+    const QUrl url = QUrl::fromUserInput( urls.at( 0 ), QDir::currentPath());
+    mMainWindow->openUrl( url );
+  }
+}
+
+void Application::restoreInstance()
+{
+  mMainWindow = new MainWindow;
+  mMainWindow->restore(1);
+}
+
+void Application::activate(const QStringList& arguments, const QString &workingDirectory)
+{
+  if (!arguments.isEmpty()) {
+    mCmdParser.parse(arguments);
+    const QStringList urls = mCmdParser.positionalArguments();
+    if (!urls.isEmpty() ) {
+      const QUrl url = QUrl::fromUserInput( urls.at( 0 ), workingDirectory );
+      mMainWindow->openUrl( url );
+    }
+  }
+
+  activateForStartLikeCall();
+}
+
+void Application::open(const QList<QUrl>& urls)
+{
+  if (!urls.isEmpty()) {
+    mMainWindow->openUrl( urls.first() );
+  }
+
+  activateForStartLikeCall();
+}
+
+int main( int argc, char **argv )
 {
   KHC::Application app(argc, argv);
   KLocalizedString::setApplicationDomain("khelpcenter5");
@@ -100,25 +124,23 @@ extern "C" int Q_DECL_EXPORT kdemain( int argc, char **argv )
   aboutData.addAuthor( QStringLiteral("Matthias Elter"), i18n("Original Author"), QStringLiteral("me@kde.org") );
   aboutData.addAuthor( QStringLiteral("Wojciech Smigaj"), i18n("Info page support"), QStringLiteral("achu@klub.chip.pl") );
   app.setWindowIcon( QIcon::fromTheme(QStringLiteral("help-browser")) );
-  aboutData.setDesktopFileName ( QStringLiteral("org.kde.Help") );
   KAboutData::setApplicationData(aboutData);
 
   QCommandLineParser *cmd = app.cmdParser();
-  KAboutData::applicationData().setupCommandLine( cmd );
+  aboutData.setupCommandLine( cmd );
   cmd->process( app );
 
+  aboutData.processCommandLine( cmd );
+
   KDBusService service( KDBusService::Unique );
 
-  KAboutData::applicationData().processCommandLine( cmd );
-
-  app.activate(app.arguments(), QDir::currentPath());
+  if (app.isSessionRestored() && KMainWindow::canBeRestored(1))
+    app.restoreInstance();
+  else
+    app.newInstance();
 
   QObject::connect( &service, &KDBusService::activateRequested, &app, &Application::activate );
-
-  if ( app.isSessionRestored() )
-  {
-     kRestoreMainWindows<MainWindow>();
-  }
+  QObject::connect( &service, &KDBusService::openRequested, &app, &Application::open );
 
   return app.exec();
 }
